/*****************************************************************************/
/* RunMSDN.cpp                              Copyright (c) Jiri Valerian 2003 */
/*---------------------------------------------------------------------------*/
/* A small tool that is used to run MSDN help for MS Visual C++ 6.0          */
/*---------------------------------------------------------------------------*/
/*   Date    Ver   Who  Comment                                              */
/* --------  ----  ---  -------                                              */
/* 13.11.03  1.00  Lad  The first version of RunMSDN.cpp                     */
/*****************************************************************************/

#include <tchar.h>
#include <stdio.h>
#include <windows.h>

// Macro for getting max. number of characters in a buffer
#define _tsize(buf)  ((sizeof(buf) / sizeof(TCHAR)) - 1)

static BOOL TimeoutExpired(DWORD dwStart, DWORD dwTimeout)
{
    return (BOOL)((GetTickCount() - dwStart) > dwTimeout);
}

static void ForceWindowFocus(HWND hWnd)
{
    POINT ClientPos = {1, 1};
    POINT CursorPos;
    DWORD dwStyle = GetWindowLong(hWnd, GWL_STYLE);

    // Ensure that the window is on top, but not topmost
    if((dwStyle & WS_CHILDWINDOW) == 0)
    {
        // If the window is in iconic form, restore it.
        if(IsIconic(hWnd))
            ShowWindow(hWnd, SW_RESTORE);

        BringWindowToTop(hWnd);
        SetForegroundWindow(hWnd);
        SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
        SetWindowPos(hWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
    }

    // Simulate the mouse click to force focus
    ClientToScreen(hWnd, &ClientPos);
    if(WindowFromPoint(ClientPos) == hWnd)
    {
        GetCursorPos(&CursorPos);
        SetCursorPos(ClientPos.x, ClientPos.y);
        mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);
        mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
        SetCursorPos(CursorPos.x, CursorPos.y);
    }
}

static int GetAppFullPath(const TCHAR * szExeName, TCHAR * szFullPath)
{
    HKEY hSubKey = NULL;
    TCHAR szKeyName[MAX_PATH] = _T("");
    int nError = ERROR_SUCCESS;

    // Open the registry key
    _stprintf(szKeyName, _T("Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\%s"), szExeName);
    nError = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKeyName, 0, KEY_QUERY_VALUE, &hSubKey);

    if(nError == ERROR_SUCCESS)
    {
        DWORD dwSize = MAX_PATH;
        nError = RegQueryValueEx(hSubKey, NULL, NULL, NULL, (LPBYTE)szFullPath, &dwSize);
    }

    if(hSubKey != NULL)
        RegCloseKey(hSubKey);

    return nError;
}

static HWND SearchForChildWindow(HWND hParent, const TCHAR * szClassName, DWORD dwStyle)
{
    TCHAR szWndClass[128];
    DWORD dwWndStyle = 0;
    HWND hFound = NULL;
    HWND hChild = GetWindow(hParent, GW_CHILD);

    while(hChild != NULL)
    {
        // Test if it's not the window
        GetClassName(hChild, szWndClass, _tsize(szWndClass));
        dwWndStyle = GetWindowLong(hChild, GWL_STYLE);

        if(!_tcsicmp(szWndClass, szClassName) && (dwWndStyle & dwStyle) == dwStyle)
            return hChild;

        // Try its child windows
        hFound = SearchForChildWindow(hChild, szClassName, dwStyle);
        if(hFound != NULL)
            return hFound;

        hChild = GetWindow(hChild, GW_HWNDNEXT);
    }

    return NULL;
}


static int InvokeMsdnSearch(HWND hWndMsdn)
{
    // Press Ctrl
    keybd_event(VK_CONTROL, 0, KEYEVENTF_EXTENDEDKEY, NULL);
    keybd_event(VK_MENU, 0, KEYEVENTF_EXTENDEDKEY, NULL);
    keybd_event(VK_F3, 0, KEYEVENTF_EXTENDEDKEY, NULL);
    keybd_event(VK_F3, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
    keybd_event(VK_MENU, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
    keybd_event(VK_CONTROL, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
    return 0;
}

static int InvokeMsdnIndex(HWND hWndMsdn, TCHAR * szWord)
{
    MSG msg;

    // Press Ctrl+Alt+F2
    keybd_event(VK_CONTROL, 0, KEYEVENTF_EXTENDEDKEY, NULL);
    keybd_event(VK_MENU, 0, KEYEVENTF_EXTENDEDKEY, NULL);
    keybd_event(VK_F2, 0, KEYEVENTF_EXTENDEDKEY, NULL);
    keybd_event(VK_F2, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
    keybd_event(VK_MENU, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
    keybd_event(VK_CONTROL, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
    
    // Wait while the window has some messages to process
    while(PeekMessage(&msg, hWndMsdn, 0, 0, PM_NOREMOVE))
        Sleep(100);

    if(szWord != NULL && *szWord != 0)
    {
        // Find the combo box with the style CBS_DROPDOWN
        HWND hCombo = SearchForChildWindow(hWndMsdn, _T("ComboBox"), CBS_DROPDOWN);

        if(hCombo != NULL)
        {
            DWORD dwCurrentThreadID = GetCurrentThreadId();
            DWORD dwThreadID = GetWindowThreadProcessId(hCombo, NULL);
            POINT ClientPos = {5, 5};
            POINT CursorPos;

            // Set the focus to the edit box
            ClientToScreen(hCombo, &ClientPos);
            GetCursorPos(&CursorPos);
            SetCursorPos(ClientPos.x, ClientPos.y);
            mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);
            mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
            SetCursorPos(CursorPos.x, CursorPos.y);

            // Write the text
            SendMessage(hCombo, WM_SETTEXT, 0, (LPARAM)szWord);

            // Press Enter
            keybd_event(VK_RETURN, 0x1C, KEYEVENTF_EXTENDEDKEY, NULL);
            keybd_event(VK_RETURN, 0x1C, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);

            // If the edit box still has a focus, press Enter once again
            // (Special case for some keywords)
            if(AttachThreadInput(dwCurrentThreadID, dwThreadID, TRUE))
            {
                TCHAR szClassName[128] = _T("");
                HWND hEdit = GetFocus();

                GetClassName(hEdit, szClassName, _tsize(szClassName));
                if(!_tcsicmp(szClassName, _T("Edit")))
                {
                    keybd_event(VK_RETURN, 0x1C, KEYEVENTF_EXTENDEDKEY, NULL);
                    keybd_event(VK_RETURN, 0x1C, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, NULL);
                }
                AttachThreadInput(dwCurrentThreadID, dwThreadID, FALSE);
            }
        }
    }
    return 0;
}

//-----------------------------------------------------------------------------
// WinMain

extern int     __argc;
extern char ** __argv;

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    DWORD dwTotalWait = 0;
    HWND hWndMsgBox = NULL;
    HWND hWndMsdn = FindWindow("wndclass_desked_gsk", NULL);

    if(__argc == 1)
    {
        MessageBox(NULL, _T("No arguments given.\n")
                         _T("Use : RunMSDN [/index:SearchedWord | /search]"),
                         _T("Error"), MB_OK | MB_ICONINFORMATION);
        return 0;
    }

    // If the MSDN window has not been found, we have to run MSDN.
    if(hWndMsdn == NULL)
    {
        PROCESS_INFORMATION pi;
        STARTUPINFO si;
        TCHAR szFullPath[MAX_PATH] = _T("");
        int nError = ERROR_SUCCESS;

        // Get the full path name
        nError = GetAppFullPath(_T("dexplore.exe"), szFullPath);

        // Retrieve the full path name of the MSDN executable
        if(nError == ERROR_SUCCESS)
        {
            ZeroMemory(&si,sizeof(STARTUPINFO));
            ZeroMemory(&pi,sizeof(PROCESS_INFORMATION));
            si.cb          = sizeof(STARTUPINFO);
            si.dwFlags     = STARTF_USESHOWWINDOW;
            si.wShowWindow = SW_SHOWMAXIMIZED;
            if(!CreateProcess(NULL, szFullPath, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
                nError = GetLastError();
        }

        // If the process ran successfully, we have to wait until
        // the process has completed its initialization
        if(nError == ERROR_SUCCESS)
        {
            while(dwTotalWait < 30000)
            {
                hWndMsgBox = FindWindow(NULL, "Help System");
                if(hWndMsgBox != NULL)
                    SendMessage(hWndMsgBox, WM_SYSCOMMAND, SC_CLOSE, 0);

                if(WaitForInputIdle(pi.hProcess, 500) == 0)
                    break;
                dwTotalWait += 500;
            }

            // Find the MSDN window again
            hWndMsdn = FindWindow("wndclass_desked_gsk", NULL);
        }
    }

    // If the window has been found, we have to focus it
    // and set the appropriate chapter (none, search, index)
    if(hWndMsdn != NULL && IsWindowVisible(hWndMsdn))
    {
        // Ensure that the window has focus
        ForceWindowFocus(hWndMsdn);

        // Invoke the apropriate action
        if(!stricmp(__argv[1], "/search"))
            InvokeMsdnSearch(hWndMsdn);
        if(!stricmp(__argv[1], "/index"))
            InvokeMsdnIndex(hWndMsdn, NULL);
        if(!strnicmp(__argv[1], "/index:", 7))
            InvokeMsdnIndex(hWndMsdn, __argv[1] + 7);
    }                       

    // Hide the message box "The MSDN collection does not exist"
    // May be still here
    hWndMsgBox = FindWindow(NULL, "Help System");
    if(hWndMsgBox != NULL)
        SendMessage(hWndMsgBox, WM_SYSCOMMAND, SC_CLOSE, 0);
    return 0;
}
